"""The SomaAPI class."""
from __future__ import annotations

import logging
from typing import Any

import aiohttp
import backoff

_LOGGER = logging.getLogger(__name__)


class Connect:
    """Represents a connection to SOMA Connect."""

    def __init__(self, host: str, port: int) -> None:
        """Initialise the connection."""
        self._host = host.removeprefix("http://")
        self._port = port
        self._url = f"http://{host}:{port}"

    @backoff.on_exception(
        backoff.expo, aiohttp.ClientError, max_tries=3, logger=_LOGGER
    )
    async def _get(
        self, endpoint: str, **kwargs: dict[str, Any]
    ) -> dict[str, Any] | None:
        """Issue a request and return the JSON."""
        url = f"{self._url}/{endpoint}"
        params = {}
        if len(kwargs) > 0:
            params = {k: v for k, v in kwargs.items()}

        async with aiohttp.ClientSession(raise_for_status=False) as session:
            async with session.get(url, params=params) as response:
                json: dict[str, Any] = await response.json()
                result = json.get("result", None)

                if result == "error":
                    json.pop("version", None)
                    json.pop("mac", None)

                return json

    async def list_devices(self) -> dict[str, Any] | None:
        """Return list of devices."""
        return await self._get("list_devices")

    async def open_shade(
        self, mac: str, **kwargs: dict[str, Any]
    ) -> dict[str, Any] | None:
        """Open the shade."""
        if mac is not None:
            return await self._get(f"open_shade/{mac}", **kwargs)

    async def close_shade(self, mac: str) -> dict[str, Any] | None:
        """Close the shade."""
        if mac is not None:
            return await self._get(f"close_shade/{mac}")

    async def stop_shade(self, mac: str) -> dict[str, Any] | None:
        """Stop the shade."""
        if mac is not None:
            return await self._get(f"stop_shade/{mac}")

    async def get_shade_state(self, mac: str) -> dict[str, Any] | None:
        """Get shade state."""
        if mac is not None:
            return await self._get(f"get_shade_state/{mac}")

    async def set_shade_position(
        self,
        mac: str,
        position: int,
        close_upwards: bool | None = False,
        morning_mode: bool | None = False,
    ) -> dict[str, Any] | None:
        """Set shade position."""
        kwargs = {}
        if mac is not None and isinstance(position, int):
            if position < 0:
                position = 0
            if position > 100:
                position = 100
            if close_upwards is True:
                kwargs["close_upwards"] = 1
            if morning_mode is True:
                kwargs["morning_mode"] = 1
            return await self._get(
                f"set_shade_position/{mac}/{str(position)}", **kwargs
            )

    async def get_battery_level(self, mac: str) -> dict[str, Any] | None:
        """Get battery level."""
        if mac is not None:
            return await self._get(f"get_battery_level/{mac}")

    async def get_light_level(self, mac: str) -> dict[str, Any] | None:
        """Get battery level."""
        if mac is not None:
            return await self._get(f"get_light_level/{mac}")



"""The Shade class."""
from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from .connect import Connect


@dataclass
class Shade:
    """Represents a SOMA shade."""

    def __init__(
        self,
        soma: Connect,
        mac: str,
        name: str,
        type: str,  # pylint: disable=W0622
        gen: str,
        light_level: int | None = None,
        position: int | None = None,
        battery_level: int | None = None,
        battery_percentage: int | None = None,
    ) -> None:
        """Initialise the Shade."""
        self._soma = soma
        self._mac = mac
        self._name = name
        self._type = type.capitalize()
        self._gen = gen
        self._light_level: int | None = light_level
        self._position: int | None = position
        self._battery_level: int | None = battery_level
        self._battery_percentage: int | None = battery_percentage

    def __str__(self) -> str:
        """Return a string representation of the shade."""
        return f"{self._name}: {self._type.capitalize()} {self._gen} ({self._mac})"

    def __repr__(self) -> str:
        """Return representation of the shade."""
        return f"<{self._name} {self._type} {self._gen} ({self._mac})>"

    @property
    def name(self) -> str:
        """Return the name of the shade."""
        return self._name

    @property
    def mac(self) -> str:
        """Return the mac address of the shade."""
        return self._mac

    @property
    def model(self) -> str:
        """Return the model of the shade."""
        return self._type

    @property
    def gen(self) -> str:
        """Return the generation of the shade."""
        return self._gen

    @property
    def light_level(self) -> int | None:
        """Return the current light level."""
        return self._light_level

    @property
    def position(self) -> int | None:
        """Return the current position."""
        return self._position

    @property
    def battery_level(self) -> int | None:
        """Return the current battery level."""
        return self._battery_level

    @property
    def battery_percentage(self) -> int | None:
        """Return the current battery level."""
        return self._battery_percentage

    async def open(self) -> dict[str, Any] | None:
        """Open this shade."""
        return await self._soma.open_shade(self._mac)

    async def close(self) -> dict[str, Any] | None:
        """Close shade."""
        return await self._soma.close_shade(self._mac)

    async def stop(self) -> dict[str, Any] | None:
        """Stop shade."""
        return await (self._soma.stop_shade(self._mac))

    async def set_position(
        self, position: int, close_upwards: bool = False, morning_mode: bool = False
    ) -> dict[str, Any] | None:
        """Set shade to specific position."""
        return await self._soma.set_shade_position(
            self._mac,
            position,
            close_upwards=close_upwards,
            morning_mode=morning_mode,
        )

    async def get_state(self) -> dict[str, Any] | None:
        """Get shade state."""
        response = await self._soma.get_shade_state(self._mac)
        return response

    async def get_battery_level(self) -> dict[str, int] | None:
        """Get battery level."""
        return await self._soma.get_battery_level(self._mac)

    async def get_light_level(self) -> dict[str, Any] | None:
        """Get light level."""
        return await self._soma.get_light_level(self._mac)
